@@ -1,4 +1,4 @@
-#   Copyright 2012-2013 OpenStack, LLC.
+#   Copyright 2012-2013 OpenStack Foundation
 #
 #   Licensed under the Apache License, Version 2.0 (the "License"); you may
 #   not use this file except in compliance with the License. You may obtain
@@ -42,6 +42,73 @@ class FakeOddballResource(fakes.FakeResource):
 
 class TestUtils(test_utils.TestCase):
 
+    def _get_test_items(self):
+        item1 = {'a': 1, 'b': 2}
+        item2 = {'a': 1, 'b': 3}
+        item3 = {'a': 2, 'b': 2}
+        item4 = {'a': 2, 'b': 1}
+        return [item1, item2, item3, item4]
+
+    def test_find_min_match_no_sort(self):
+        items = self._get_test_items()
+        sort_str = None
+        flair = {}
+        expect_items = items
+        self.assertEqual(
+            expect_items,
+            list(utils.find_min_match(items, sort_str, **flair)),
+        )
+
+    def test_find_min_match_no_flair(self):
+        items = self._get_test_items()
+        sort_str = 'b'
+        flair = {}
+        expect_items = [items[3], items[0], items[2], items[1]]
+        self.assertEqual(
+            expect_items,
+            utils.find_min_match(items, sort_str, **flair),
+        )
+
+    def test_find_min_match_a2(self):
+        items = self._get_test_items()
+        sort_str = 'b'
+        flair = {'a': 2}
+        expect_items = [items[3], items[2]]
+        self.assertEqual(
+            expect_items,
+            utils.find_min_match(items, sort_str, **flair),
+        )
+
+    def test_find_min_match_b2(self):
+        items = self._get_test_items()
+        sort_str = 'b'
+        flair = {'b': 2}
+        expect_items = [items[0], items[2], items[1]]
+        self.assertEqual(
+            expect_items,
+            utils.find_min_match(items, sort_str, **flair),
+        )
+
+    def test_find_min_match_b5(self):
+        items = self._get_test_items()
+        sort_str = 'b'
+        flair = {'b': 5}
+        expect_items = []
+        self.assertEqual(
+            expect_items,
+            utils.find_min_match(items, sort_str, **flair),
+        )
+
+    def test_find_min_match_a2_b2(self):
+        items = self._get_test_items()
+        sort_str = 'b'
+        flair = {'a': 2, 'b': 2}
+        expect_items = [items[2]]
+        self.assertEqual(
+            expect_items,
+            utils.find_min_match(items, sort_str, **flair),
+        )
+
     def test_get_password_good(self):
         with mock.patch("getpass.getpass", return_value=PASSWORD):
             mock_stdin = mock.Mock()
@@ -74,33 +141,26 @@ class TestUtils(test_utils.TestCase):
                               utils.get_password,
                               mock_stdin)
 
-    def get_test_items(self):
-        item1 = {'a': 1, 'b': 2}
-        item2 = {'a': 1, 'b': 3}
-        item3 = {'a': 2, 'b': 2}
-        item4 = {'a': 2, 'b': 1}
-        return [item1, item2, item3, item4]
-
     def test_sort_items_with_one_key(self):
-        items = self.get_test_items()
+        items = self._get_test_items()
         sort_str = 'b'
         expect_items = [items[3], items[0], items[2], items[1]]
         self.assertEqual(expect_items, utils.sort_items(items, sort_str))
 
     def test_sort_items_with_multiple_keys(self):
-        items = self.get_test_items()
+        items = self._get_test_items()
         sort_str = 'a,b'
         expect_items = [items[0], items[1], items[3], items[2]]
         self.assertEqual(expect_items, utils.sort_items(items, sort_str))
 
     def test_sort_items_all_with_direction(self):
-        items = self.get_test_items()
+        items = self._get_test_items()
         sort_str = 'a:desc,b:desc'
         expect_items = [items[2], items[3], items[1], items[0]]
         self.assertEqual(expect_items, utils.sort_items(items, sort_str))
 
     def test_sort_items_some_with_direction(self):
-        items = self.get_test_items()
+        items = self._get_test_items()
         sort_str = 'a,b:desc'
         expect_items = [items[1], items[0], items[2], items[3]]
         self.assertEqual(expect_items, utils.sort_items(items, sort_str))
@@ -116,66 +176,26 @@ class TestUtils(test_utils.TestCase):
         self.assertEqual(expect_items, utils.sort_items(items, sort_str))
 
     def test_sort_items_with_empty_key(self):
-        items = self.get_test_items()
+        items = self._get_test_items()
         sort_srt = ''
         self.assertEqual(items, utils.sort_items(items, sort_srt))
         sort_srt = None
         self.assertEqual(items, utils.sort_items(items, sort_srt))
 
     def test_sort_items_with_invalid_key(self):
-        items = self.get_test_items()
+        items = self._get_test_items()
         sort_str = 'c'
         self.assertRaises(exceptions.CommandError,
                           utils.sort_items,
                           items, sort_str)
 
     def test_sort_items_with_invalid_direction(self):
-        items = self.get_test_items()
+        items = self._get_test_items()
         sort_str = 'a:bad_dir'
         self.assertRaises(exceptions.CommandError,
                           utils.sort_items,
                           items, sort_str)
 
-    @mock.patch.object(time, 'sleep')
-    def test_wait_for_status_ok(self, mock_sleep):
-        # Tests the normal flow that the resource is status=active
-        resource = mock.MagicMock(status='ACTIVE')
-        status_f = mock.Mock(return_value=resource)
-        res_id = str(uuid.uuid4())
-        self.assertTrue(utils.wait_for_status(status_f, res_id,))
-        mock_sleep.assert_not_called()
-
-    @mock.patch.object(time, 'sleep')
-    def test_wait_for_status_ok__with_overrides(self, mock_sleep):
-        # Tests the normal flow that the resource is status=complete
-        resource = mock.MagicMock(my_status='COMPLETE')
-        status_f = mock.Mock(return_value=resource)
-        res_id = str(uuid.uuid4())
-        self.assertTrue(utils.wait_for_status(status_f, res_id,
-                                              status_field='my_status',
-                                              success_status=['complete']))
-        mock_sleep.assert_not_called()
-
-    @mock.patch.object(time, 'sleep')
-    def test_wait_for_status_error(self, mock_sleep):
-        # Tests that we fail if the resource is status=error
-        resource = mock.MagicMock(status='ERROR')
-        status_f = mock.Mock(return_value=resource)
-        res_id = str(uuid.uuid4())
-        self.assertFalse(utils.wait_for_status(status_f, res_id))
-        mock_sleep.assert_not_called()
-
-    @mock.patch.object(time, 'sleep')
-    def test_wait_for_status_error_with_overrides(self, mock_sleep):
-        # Tests that we fail if the resource is my_status=failed
-        resource = mock.MagicMock(my_status='FAILED')
-        status_f = mock.Mock(return_value=resource)
-        res_id = str(uuid.uuid4())
-        self.assertFalse(utils.wait_for_status(status_f, res_id,
-                                               status_field='my_status',
-                                               error_status=['failed']))
-        mock_sleep.assert_not_called()
-
     @mock.patch.object(time, 'sleep')
     def test_wait_for_delete_ok(self, mock_sleep):
         # Tests the normal flow that the resource is deleted with a 404 coming
@@ -234,6 +254,46 @@ class TestUtils(test_utils.TestCase):
                                               exception_name=['Exception']))
         mock_sleep.assert_not_called()
 
+    @mock.patch.object(time, 'sleep')
+    def test_wait_for_status_ok(self, mock_sleep):
+        # Tests the normal flow that the resource is status=active
+        resource = mock.MagicMock(status='ACTIVE')
+        status_f = mock.Mock(return_value=resource)
+        res_id = str(uuid.uuid4())
+        self.assertTrue(utils.wait_for_status(status_f, res_id,))
+        mock_sleep.assert_not_called()
+
+    @mock.patch.object(time, 'sleep')
+    def test_wait_for_status_ok_with_overrides(self, mock_sleep):
+        # Tests the normal flow that the resource is status=complete
+        resource = mock.MagicMock(my_status='COMPLETE')
+        status_f = mock.Mock(return_value=resource)
+        res_id = str(uuid.uuid4())
+        self.assertTrue(utils.wait_for_status(status_f, res_id,
+                                              status_field='my_status',
+                                              success_status=['complete']))
+        mock_sleep.assert_not_called()
+
+    @mock.patch.object(time, 'sleep')
+    def test_wait_for_status_error(self, mock_sleep):
+        # Tests that we fail if the resource is status=error
+        resource = mock.MagicMock(status='ERROR')
+        status_f = mock.Mock(return_value=resource)
+        res_id = str(uuid.uuid4())
+        self.assertFalse(utils.wait_for_status(status_f, res_id))
+        mock_sleep.assert_not_called()
+
+    @mock.patch.object(time, 'sleep')
+    def test_wait_for_status_error_with_overrides(self, mock_sleep):
+        # Tests that we fail if the resource is my_status=failed
+        resource = mock.MagicMock(my_status='FAILED')
+        status_f = mock.Mock(return_value=resource)
+        res_id = str(uuid.uuid4())
+        self.assertFalse(utils.wait_for_status(status_f, res_id,
+                                               status_field='my_status',
+                                               error_status=['failed']))
+        mock_sleep.assert_not_called()
+
     def test_build_kwargs_dict_value_set(self):
         self.assertEqual({'arg_bla': 'bla'},
                          utils.build_kwargs_dict('arg_bla', 'bla'))
@@ -244,6 +304,12 @@ class TestUtils(test_utils.TestCase):
     def test_build_kwargs_dict_value_empty_str(self):
         self.assertEqual({}, utils.build_kwargs_dict('arg_bla', ''))
 
+    def test_is_ascii_bytes(self):
+        self.assertFalse(utils.is_ascii(b'\xe2'))
+
+    def test_is_ascii_string(self):
+        self.assertFalse(utils.is_ascii(u'\u2665'))
+
 
 class NoUniqueMatch(Exception):
     pass
@@ -371,6 +437,36 @@ class TestFindResource(test_utils.TestCase):
         self.manager.get.assert_called_with(self.name)
         self.manager.find.assert_called_with(name=self.name)
 
+    def test_find_resource_silly_resource_no_unique_match(self):
+        # We need a resource with no resource_class for this test, start fresh
+        self.manager = mock.Mock()
+        self.manager.get = mock.Mock(side_effect=Exception('Boom!'))
+        self.manager.find = mock.Mock(
+            side_effect=AttributeError(
+                "'Controller' object has no attribute 'find'",
+            )
+        )
+        silly_resource = FakeOddballResource(
+            None,
+            {'id': '12345', 'name': self.name},
+            loaded=True,
+        )
+        silly_resource_same = FakeOddballResource(
+            None,
+            {'id': 'abcde', 'name': self.name},
+            loaded=True,
+        )
+        self.manager.list = mock.Mock(return_value=[silly_resource,
+                                                    silly_resource_same])
+        result = self.assertRaises(exceptions.CommandError,
+                                   utils.find_resource,
+                                   self.manager,
+                                   self.name)
+        self.assertEqual("More than one resource exists "
+                         "with the name or ID 'legos'.", str(result))
+        self.manager.get.assert_called_with(self.name)
+        self.manager.find.assert_called_with(name=self.name)
+
     def test_format_dict(self):
         expected = "a='b', c='d', e='f'"
         self.assertEqual(expected,
